import { RenderSelectOptionArgType } from '../types';
import { fireEvent, render, screen, waitFor } from '@/test-utils/index';
import React from 'react';

import { NsSelect, NsSelectOption } from '../..';

const renderSelectOption = (option: RenderSelectOptionArgType) => {
    return (
        <NsSelectOption
            key={option.value}
            value={option.value}
            index={option.index ?? -1}
            label={option.label}
            onClick={option.onClick}
        />
    );
};
const createOptionTestId = (value: string) => `option-${value}`;
const selectCommonProps = {
    selected: { label: 'Option 1', value: 'Option1', index: 0 },
    options: [
        { value: 'Option1', label: 'Option 1' },
        { value: 'Option2', label: 'Option 2' },
        { value: 'Option3', label: 'Option 3' },
    ],
    onSelect: jest.fn(),
    RenderOption: renderSelectOption,
};
describe('NsSelect', () => {
    const onSelect = jest.fn();
    const selected = { label: '', value: '' };

    const options = [
        { value: 'Option1', label: 'Option 1' },
        { value: 'Option2', label: 'Option 2' },
        { value: 'Option3', label: 'Option 3' },
    ];

    it('1. renders the placeholder text when no value is selected', () => {
        render(
            <NsSelect
                placeholder="Select an option"
                selected={selected}
                onSelect={onSelect}
                options={options}
                RenderOption={renderSelectOption}
            />
        );

        expect(screen.getByText('Select an option')).toBeInTheDocument();
    });

    it('2. clicking outside the select container closes the dropdown', async () => {
        const { getByTestId, queryByTestId, container } = render(
            <div>
                <button className="extra"></button>
                <NsSelect {...selectCommonProps} />
            </div>
        );
        const selectButton = getByTestId('select-head');
        fireEvent.mouseDown(selectButton);
        const outsideElement = container.querySelector('button.extra');
        if (outsideElement) fireEvent.click(outsideElement);
        // check if the dropdown is closed
        expect(queryByTestId(createOptionTestId(selectCommonProps.options[1].value))).toBeNull();
    });

    it('3. renders the selected value label', () => {
        render(<NsSelect {...selectCommonProps} />);

        expect(screen.getByText(selectCommonProps.selected.label)).toBeInTheDocument();
    });

    it('4. clicking on select button opens the dropdown', () => {
        const { getByTestId, getByText } = render(<NsSelect {...selectCommonProps} />);
        const selectButton = getByTestId('select-head');
        fireEvent.mouseDown(selectButton);

        // Check if the dropdown is opened (isOpen state is true)
        const dropdownList = getByTestId('select-menu-list');
        expect(dropdownList).toBeInTheDocument();

        // Check if the options are rendered in the dropdown
        selectCommonProps.options.slice(1).forEach((option) => {
            const optionElement = getByText(option.label);
            expect(optionElement).toBeInTheDocument();
        });
    });

    it('5. disabling the select component prevents interaction', () => {
        const { getByTestId, queryByTestId } = render(<NsSelect {...selectCommonProps} disabled />);
        const selectButton = getByTestId('select-head');
        fireEvent.mouseDown(selectButton);

        // Check if the dropdown does not open (select list is not present)
        const dropdownList = queryByTestId('select-menu-list');
        expect(dropdownList).toBeNull();

        expect(selectCommonProps.onSelect).not.toHaveBeenCalled();
    });

    it('6. menu closes on mouse leave when closeOnMouseLeave is set to true', async () => {
        const { getByTestId } = render(<NsSelect {...selectCommonProps} closeOnMouseLeave />);

        const selectHead = getByTestId('select-head');
        fireEvent.mouseDown(selectHead);
        const selectList = screen.queryByTestId('select-menu-list');
        const option1 = screen.queryByTestId(createOptionTestId(selectCommonProps.options[0].value));
        const option2 = screen.queryByTestId(createOptionTestId(selectCommonProps.options[1].value));
        const option3 = screen.queryByTestId(createOptionTestId(selectCommonProps.options[2].value));
        if (selectList) fireEvent.mouseLeave(selectList);

        await waitFor(() => {
            expect(option1).not.toBeInTheDocument();
            expect(option2).not.toBeInTheDocument();
            expect(option3).not.toBeInTheDocument();
        });
    });

    it('7. select does not open when disabled', () => {
        render(<NsSelect {...selectCommonProps} disabled />);

        const selectHead = screen.getByTestId('select-head');
        fireEvent.mouseDown(selectHead);

        expect(screen.queryByTestId('select-menu-list')).not.toBeInTheDocument();
    });

    it('8. opens the select on "ArrowDown" key press', () => {
        render(<NsSelect {...selectCommonProps} />);

        const selectHead = screen.getByTestId('select-head');
        fireEvent.mouseDown(selectHead, { key: 'arrowDown' });

        expect(selectHead).toBeInTheDocument();
    });

    it('9. selects the next option on arrow right click', () => {
        const { getByTestId, container } = render(<NsSelect {...selectCommonProps} hasArrows />);

        const selectHead = getByTestId('select-head');
        fireEvent.mouseDown(selectHead);

        const rightArrow = container.querySelector('.RightArrow');
        if (rightArrow) fireEvent.click(rightArrow);

        expect(selectCommonProps.onSelect).toHaveBeenCalledWith({ ...selectCommonProps.options[1], index: 1 });
    });

    it('10. selects the previous option on arrow left click', () => {
        const { getByTestId } = render(<NsSelect {...selectCommonProps} hasArrows reverseArrowControls />);

        const selectHead = getByTestId('select-head');
        fireEvent.mouseDown(selectHead);
        const option3 = getByTestId(createOptionTestId(selectCommonProps.options[2].value));
        fireEvent.click(option3);

        const leftArrow = document.getElementsByClassName('LeftArrow')[0];
        fireEvent.click(leftArrow);

        expect(selectCommonProps.onSelect).toHaveBeenCalledWith({ ...selectCommonProps.options[1], index: 1 });
    });

    it('11. renders an error message with red styles', () => {
        const errorMessage = 'Invalid value';

        const { getByText } = render(<NsSelect {...selectCommonProps} hasArrows errorMessage={errorMessage} />);

        const errorText = getByText(errorMessage);
        expect(errorText).toBeInTheDocument();
    });
    it('12. selecting an option calls the onSelect callback', () => {
        const { getByTestId } = render(<NsSelect {...selectCommonProps} hasArrows />);

        const selectHead = getByTestId('select-head');
        fireEvent.mouseDown(selectHead);

        const option1 = getByTestId(createOptionTestId(selectCommonProps.options[2].value));
        fireEvent.click(option1);

        expect(selectCommonProps.onSelect).toHaveBeenCalledWith({ ...selectCommonProps.options[2], index: 2 });
    });
});
